@@ -1,11 +1,12 @@
 /* cipher.c  -	cipher dispatcher
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -178,6 +177,31 @@ setup_cipher_table(void)
 	BUG();
     i++;
 
+#ifdef USE_CAMELLIA
+    cipher_table[i].algo = CIPHER_ALGO_CAMELLIA128;
+    cipher_table[i].name = camellia_get_info( cipher_table[i].algo,
+					      &cipher_table[i].keylen,
+					      &cipher_table[i].blocksize,
+					      &cipher_table[i].contextsize,
+					      &cipher_table[i].setkey,
+					      &cipher_table[i].encrypt,
+					      &cipher_table[i].decrypt     );
+    if( !cipher_table[i].name )
+	BUG();
+    i++;
+    cipher_table[i].algo = CIPHER_ALGO_CAMELLIA256;
+    cipher_table[i].name = camellia_get_info( cipher_table[i].algo,
+					      &cipher_table[i].keylen,
+					      &cipher_table[i].blocksize,
+					      &cipher_table[i].contextsize,
+					      &cipher_table[i].setkey,
+					      &cipher_table[i].encrypt,
+					      &cipher_table[i].decrypt     );
+    if( !cipher_table[i].name )
+	BUG();
+    i++;
+#endif
+
 #ifdef USE_IDEA
     cipher_table[i].algo = CIPHER_ALGO_IDEA;
     cipher_table[i].name = idea_get_info( cipher_table[i].algo,
@@ -389,10 +413,10 @@ cipher_open( int algo, int mode, int secure )
 
     /* ? perform selftest here and mark this with a flag in cipher_table ? */
 
-    hd = secure ? m_alloc_secure_clear( sizeof *hd
+    hd = secure ? xmalloc_secure_clear( sizeof *hd
 					+ cipher_table[i].contextsize
 					- sizeof(PROPERLY_ALIGNED_TYPE) )
-		: m_alloc_clear( sizeof *hd + cipher_table[i].contextsize
+		: xmalloc_clear( sizeof *hd + cipher_table[i].contextsize
 					   - sizeof(PROPERLY_ALIGNED_TYPE)  );
     hd->algo = algo;
     hd->blocksize = cipher_table[i].blocksize;
@@ -421,7 +445,7 @@ cipher_open( int algo, int mode, int secure )
 void
 cipher_close( CIPHER_HANDLE c )
 {
-    m_free(c);
+    xfree(c);
 }
 
 
@@ -536,7 +560,25 @@ do_cfb_encrypt( CIPHER_HANDLE c, byte *outbuf, byte *inbuf, unsigned nbytes )
 	    *outbuf++ = (*ivp++ ^= *inbuf++);
     }
 
-    /* now we can process complete blocks */
+    /* Now we can process complete blocks. */
+#if 0 
+    /* Experimental code.  We may only use this for standard CFB
+       because for Phil's mode we need to save the IV of before the
+       last encryption - we don't want to do this in tghe fasf CFB
+       encryption routine.  */
+    if (c->algo == CIPHER_ALGO_AES
+        && nbytes >= blocksize 
+        && c->mode != CIPHER_MODE_PHILS_CFB) {
+        size_t n;
+
+	memcpy( c->lastiv, c->iv, blocksize );
+        n = (nbytes / blocksize) * blocksize;
+        rijndael_cfb_encrypt (&c->context.c, c->iv, outbuf, inbuf, n);
+        inbuf  += n;
+        outbuf += n;
+	nbytes -= n;
+    }
+#endif
     while( nbytes >= blocksize ) {
 	int i;
 	/* encrypt the IV (and save the current one) */
